import unicodedata

from bs4 import BeautifulSoup

from eveuniverse.core import evexml

from allianceauth.services.hooks import get_extension_logger
from app_utils.logging import LoggerAddTag

from .. import __title__

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


def eve_xml_to_discord_markup(xml_doc: str) -> str:
    """Converts Eve Online xml to Discord markup."""
    xml_doc = unicodedata.normalize("NFKC", xml_doc)
    xml_doc = evexml.remove_loc_tag(xml_doc)
    soup = BeautifulSoup(xml_doc, "html.parser")
    for element in soup.find_all("br"):
        element.replace_with("\n")
    for element in soup.find_all("b"):
        element.replace_with(f"**{element.string}**")
    for element in soup.find_all("i"):
        element.replace_with(f"_{element.string}_")
    for element in soup.find_all("u"):
        element.replace_with(f"__{element.string}__")
    for element in soup.find_all("a"):
        text = element.string
        url = evexml.eve_link_to_url(element["href"])
        if url:
            element.replace_with(f"[{text}]({url})")
        else:
            element.replace_with(f"**{text}**")
    markup_text = soup.get_text()
    logger.debug("Markdown text:\n%s", markup_text)
    return markup_text



# Generated by Django 4.0.3 on 2022-03-02 17:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("memberaudit", "0007_add_exports_permission"),
        ("mailrelay", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="relayconfig",
            name="mails_sent",
            field=models.ManyToManyField(
                editable=False,
                help_text="Latest mails that have already been sent.",
                related_name="+",
                to="memberaudit.charactermail",
            ),
        ),
    ]



from discordproxy.exceptions import DiscordProxyException

from django.contrib import messages
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.decorators import login_required
from django.shortcuts import redirect

from allianceauth.services.hooks import get_extension_logger
from app_utils.logging import LoggerAddTag

from . import __title__
from .app_settings import MAILRELAY_DISCORD_USER_TIMEOUT
from .models import DiscordChannel

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


@login_required
@staff_member_required
def admin_update_discord_channels(request):
    try:
        channels_count = DiscordChannel.objects.sync(
            timeout=MAILRELAY_DISCORD_USER_TIMEOUT
        )
        messages.success(
            request, f"Successfully updated {channels_count} channels from Discord."
        )
    except DiscordProxyException as ex:
        logger.error("Failed to fetch channels from Discord", exc_info=True)
        messages.warning(request, f"Failed to fetch channels from Discord: {ex}")
    return redirect("admin:mailrelay_relayconfig_changelist")



# Generated by Django 3.2.9 on 2022-01-02 20:11

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("memberaudit", "0006_add_current_ship"),
    ]

    operations = [
        migrations.CreateModel(
            name="DiscordCategory",
            fields=[
                ("id", models.BigIntegerField(primary_key=True, serialize=False)),
                ("name", models.CharField(db_index=True, max_length=100)),
                ("last_update_at", models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name="DiscordChannel",
            fields=[
                ("id", models.BigIntegerField(primary_key=True, serialize=False)),
                ("name", models.CharField(db_index=True, max_length=100)),
                ("last_update_at", models.DateTimeField(auto_now=True)),
                (
                    "category",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.SET_DEFAULT,
                        related_name="channels",
                        to="mailrelay.discordcategory",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="RelayConfig",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "is_enabled",
                    models.BooleanField(
                        default=True,
                        help_text="Toogle for activating or deactivating relaying mails.",
                    ),
                ),
                (
                    "last_service_run_at",
                    models.DateTimeField(
                        default=None,
                        editable=False,
                        help_text="Time of last successful service run.",
                        null=True,
                    ),
                ),
                (
                    "mail_category",
                    models.CharField(
                        choices=[
                            ("AL", "All mails"),
                            ("AM", "Alliance mails"),
                            ("CM", "Corporation mails"),
                        ],
                        help_text="Category of mails that you want to relay to Discord.",
                        max_length=2,
                    ),
                ),
                (
                    "ping_type",
                    models.CharField(
                        choices=[
                            ("PN", "(none)"),
                            ("PH", "@here"),
                            ("PE", "@everybody"),
                        ],
                        default="PN",
                        help_text="Option to ping every member of the channel.",
                        max_length=2,
                        verbose_name="channel pings",
                    ),
                ),
                (
                    "character",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="memberaudit.character",
                    ),
                ),
                (
                    "discord_channel",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="mailrelay.discordchannel",
                    ),
                ),
                (
                    "mails_sent",
                    models.ManyToManyField(
                        editable=False,
                        help_text="Latest mails that have already been sent.",
                        related_name="_mailrelay_relayconfig_mails_sent_+",
                        to="memberaudit.CharacterMail",
                    ),
                ),
            ],
        ),
    ]



from typing import List


def chunks_by_lines(full_text: str, max_lengths: int) -> List[str]:
    """Converts text into chunks not exceeding max_lengths and splity by newline."""
    parts = list()
    partial_text = ""
    for line in full_text.splitlines(keepends=True):
        if len(partial_text + line) > max_lengths:
            parts.append(partial_text)
            partial_text = ""
        partial_text += line
    if partial_text:
        parts.append(partial_text)
    return parts



import os

from setuptools import find_packages, setup

from mailrelay import __version__

# read the contents of your README file
this_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name="aa-mailrelay",
    version=__version__,
    packages=find_packages(),
    include_package_data=True,
    license="MIT",
    description="An app for relaying Eve mails to Discord",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/ErikKalkoken/aa-mailrelay",
    author="Erik Kalkoken",
    author_email="kalkoken87@gmail.com",
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 3.2",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",  # mailrelay license
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    python_requires="~=3.7",
    install_requires=[
        "allianceauth>=2.9",
        "aa-memberaudit",
        "django-eveuniverse>=0.14",
        "discordproxy>=0.4",
        "allianceauth-app-utils>=1.12",
    ],
)
